﻿using Proto;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    private static void Main()
    {
        RootContext rootContext = new RootContext(new ActorSystem());

        int count = 0;

        Props props = Props.FromFunc
        (
            context =>
            {
                switch(context.Message)
                {
                    case Started _ :

                        Console.WriteLine($"{DateTime.Now:HH:mm:ss} 시작");

                        context.SetReceiveTimeout(TimeSpan.FromSeconds(1));

                        break;

                    case ReceiveTimeout _ :

                        count++;

                        Console.WriteLine($"{DateTime.Now:HH:mm:ss} 수신 타임아웃 : {count}");

                        break;

                    case NoInfluenceMessage _ :

                        Console.WriteLine($"{DateTime.Now:HH:mm:ss} 영향을 주지 않는 메시지 수신");

                        break;

                    case string message :

                        Console.WriteLine($"{DateTime.Now:HH:mm:ss} 메시지 수신 : {message}");

                        break;
                }

                return Task.CompletedTask;
            }
        );

        PID pid = rootContext.Spawn(props);

        for(int i = 0; i < 6; i++)
        {
            rootContext.Send(pid, "안녕하세요.");

            Thread.Sleep(500);
        }

        Console.WriteLine("영향을 주지 않는 메시지를 보내기 위해서 RETURN 키를 눌러 주세요.");

        Console.ReadLine();

        for(int i = 0; i < 6; i++)
        {
            rootContext.Send(pid, new NoInfluenceMessage());

            Thread.Sleep(500);
        }

        Console.WriteLine("타임아웃을 취소하기 위한 메시지를 보내기 위해서 RETURN 키를 눌러 주세요.");

        Console.ReadLine();

        rootContext.Send(pid, "취소");

        Console.WriteLine("종료하기 위해서 RETURN 키를 눌러 주세요.");

        Console.ReadLine();
    }

    #endregion
}